// Copyright 2023 BlindAssassin111 (@blindassassin111)
// SPDX-License-Identifier: GPL-2.0-or-later

#include QMK_KEYBOARD_H

// Defines names for use in layer keycodes and the keymap
enum layer_names {
    _BASE,
    _FN
};

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
    /*
    * ┌──┐┌──┐┌──┐┌──┐  ┌──┐ ┌──┐┌──┐┌──┐┌──┐ ┌──┐┌──┐    ┌──┐┌──┐ ┌──┐┌──┐┌──┐┌──┐ ┌──┐ ┌──┐┌──┐
    * │0A││0B││0C││0D│  │0E│ │0F││0G││0H││0I│ │0J││0K│    │6A││6B│ │6D││6E││6F││6G│ │6H│ │6I││6J│
    * └──┘└──┘└──┘└──┘  └──┘ └──┘└──┘└──┘└──┘ └──┘└──┘    └──┘└──┘ └──┘└──┘└──┘└──┘ └──┘ └──┘└──┘
    * ┌──┐┌──┐┌──┐┌──┐  ┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐    ┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐ ┌──┐┌──┐
    * │1A││1B││1C││1D│  │1E││1F││1G││1H││1I││1J││1K│    │7A││7B││7C││7D││7E││7F││7G││7H│ │7I││7J│
    * └──┘└──┘└──┘└──┘  └──┘└──┘└──┘└──┘└──┘└──┘└──┘    └──┘└──┘└──┘└──┘└──┘└──┘└──┘└──┘ └──┘└──┘
    * ┌──┐┌──┐┌──┐┌──┐  ┌────┐┌──┐┌──┐┌──┐┌──┐┌──┐    ┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌────┐ ┌──┐┌──┐
    * │2A││2B││2C││2D│  │2E  ││2F││2G││2H││2I││2J│    │8A││8B││8C││8D││8E││8F││8G││8H  │ │8I││8J│
    * └──┘└──┘└──┘└──┘  └────┘└──┘└──┘└──┘└──┘└──┘    └──┘└──┘└──┘└──┘└──┘└──┘└──┘└────┘ └──┘└──┘
    * ┌──┐┌──┐┌──┐┌──┐  ┌─────┐┌──┐┌──┐┌──┐┌──┐┌──┐    ┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐┌───┐ ┌──┐┌──┐
    * │3A││3B││3C││3D│  │3E   ││3F││3G││3H││3I││3J│    │9A││9B││9C││9D││9E││9F││9G││9H │ │9I││9J│
    * └──┘└──┘└──┘└──┘  └─────┘└──┘└──┘└──┘└──┘└──┘    └──┘└──┘└──┘└──┘└──┘└──┘└──┘└───┘ └──┘└──┘
    * ┌──┐┌──┐┌──┐┌──┐  ┌───┐┌──┐┌──┐┌──┐┌──┐┌──┐┌──┐    ┌──┐┌──┐┌──┐┌──┐┌──┐┌─────┐┌──┐
    * │4A││4B││4C││4D│  │4E ││4F││4G││4H││4I││4J││4K│    │AB││AC││AD││AE││AF││AG   ││AH│ ┌──┐
    * └──┘└──┘└──┘└──┘  └───┘└──┘└──┘└──┘└──┘└──┘└──┘    └──┘└──┘└──┘└──┘└──┘└─────┘└──┘ │AI│
    * ┌──┐┌──┐┌──┐┌──┐  ┌───┐┌───┐┌───┐┌──┐┌───────┐    ┌───────┐┌──┐┌───┐┌───┐┌───┐     └──┘
    * │5A││5B││5C││5D│  │5E ││5F ││5H ││5I││5J     │    │BC     ││BD││BE ││BF ││BG │ ┌──┐┌──┐┌──┐
    * └──┘└──┘└──┘└──┘  └───┘└───┘└───┘└──┘└───────┘    └───────┘└──┘└───┘└───┘└───┘ │BH││BI││BJ│
    *                                                                                └──┘└──┘└──┘
    */
    [_BASE] = LAYOUT(
    KC_MUTE,KC_MPRV,KC_MPLY,KC_MNXT,    KC_ESC,  KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F11,  KC_F12,           KC_PSCR, KC_SCRL, KC_PAUS,
    KC_NUM, KC_PSLS,KC_PAST,KC_PMNS,    KC_GRV,  KC_1,    KC_2,    KC_3,    KC_4,    KC_5,    KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS, KC_EQL,  KC_BSPC, KC_DEL,  KC_HOME, KC_INS,
    KC_P7,  KC_P8,  KC_P9,  KC_PPLS,    KC_TAB,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,    KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_LBRC, KC_RBRC, KC_BSLS,          KC_END,  KC_DEL,
    KC_P4,  KC_P5,  KC_P6,  KC_PEQL,    KC_CAPS, KC_A,    KC_S,    KC_D,    KC_F,    KC_G,    KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT, KC_NUHS, KC_ENT,           KC_PGUP, KC_PGDN,
    KC_P1,  KC_P2,  KC_P3,  KC_PENT,    KC_LSFT, KC_NUBS, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, KC_RSFT, MO(_FN),          KC_UP,
    KC_P0,  KC_P0,  KC_P0,  KC_PDOT,    KC_LCTL, KC_LGUI, KC_LALT, KC_MUTE, KC_SPC,           KC_SPC,  KC_APP,  KC_RALT, KC_RGUI, KC_RCTL,                            KC_LEFT, KC_DOWN, KC_RIGHT),

    [_FN] = LAYOUT(
    _______, _______, _______, _______,      _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______, QK_BOOT,
    _______, _______, _______, _______,      _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
    _______, _______, _______, _______,      _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______,
    _______, _______, _______, _______,      _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______, _______,
    _______, _______, _______, _______,      _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,          _______,
    _______, _______, _______, _______,      _______, _______, _______, _______, _______,          _______, _______, _______, _______, _______,                            _______, _______, _______)
};
